
# q-expansion of newform 546.4.a.n, downloaded from the LMFDB on 28 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 546
weight = 4
poly_data = [-224, -153, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[0, 2, 0], 1], [[-104, -1, 1], 6]]

hecke_ring_character_values = None
aps_data = [[-2, 0, 0], [3, 0, 0], [4, 0, -1], [7, 0, 0], [6, -2, 1], [-13, 0, 0], [30, 3, 1], [28, -3, -5], [96, 5, -1], [46, -3, 13], [-68, 2, 4], [66, -5, -15], [184, -5, 22], [-16, 7, 27], [94, 19, -16], [238, -10, 4], [-90, -1, -20], [-162, -15, 25], [-164, 4, 30], [102, -39, 24], [-230, 31, 11], [408, 18, -38], [562, 3, -14], [532, 11, -32], [510, 40, 4], [346, -42, 86], [52, -33, 5], [720, -30, 74], [646, -29, 79], [710, 38, 16], [472, 30, 98], [892, 67, -103], [212, 112, -57], [604, -60, -84], [300, -41, 94], [-32, 67, -131], [146, -73, 189], [-1584, 18, 34], [1866, -38, 3], [-502, -28, -196], [-844, -118, 172], [-802, 62, -12], [-348, -61, -265], [-1822, -58, 78], [-1600, 33, 94], [964, 109, -169], [-2120, 65, 73], [-252, 40, 194], [-106, 57, -350], [-5086, -4, -156], [-1426, -136, -214], [-1690, 101, -40], [-1850, 20, -12], [1740, -137, 165], [-4762, 0, -192], [468, 10, 308], [482, 68, 16], [-3848, 198, 202], [3446, -120, -276], [-4752, -31, -16], [-1884, 208, -40], [-1644, 39, 408], [-660, -320, -44], [2724, -104, 462], [270, 44, -38], [-40, 165, 554], [-732, -116, -142], [-4414, -171, -269], [1780, 446, -616], [86, -90, -300], [1280, -215, -420], [-5930, 161, 112], [1848, -2, -164], [2490, -44, 594], [-1664, 108, -368], [302, -112, -277], [710, 94, 124], [-3114, -322, 628], [2948, -45, -184], [-3078, 125, 133], [-2284, -395, -357], [7634, 372, -100], [4114, -431, 490], [-1018, -56, 374], [-2516, -261, -375], [11048, 238, -186], [-7664, 466, 331], [-42, -396, -142], [-380, 486, -463], [-6908, -251, -203], [-1052, 515, 285], [-1962, 506, 785], [2492, -186, 1424], [-5040, -106, -690], [668, -70, 1044], [7172, 400, -86], [-14492, 134, -351], [1410, -613, -513], [-4684, 32, 1036], [4518, 119, -233], [11380, 44, 616], [-6044, -281, 814], [288, 317, 569], [-12730, -182, -1080], [15900, -252, -116], [-10878, 58, 508], [-5638, 475, 846], [-13060, 515, 624], [-1160, 611, -783], [-6322, -884, 950], [-13628, 195, -1383], [5042, 289, 1399], [-12252, 190, 641], [2600, -217, -37], [17752, -109, -47], [6610, -136, 288], [-736, -27, -107], [-20464, 148, -192], [8630, 475, -777], [4220, -600, -490], [-16474, -62, -1076], [-5018, -33, 1703], [-15914, -474, -424], [2398, -756, 971], [-84, -188, -1720], [9382, 386, 1416], [19786, -244, 884], [5676, -280, -158], [940, 387, -95], [-7886, 302, -1714], [9044, 512, -1730], [7646, 167, 842], [-15072, 1156, 168], [11378, -162, -320], [1720, 39, -694], [-24346, -501, 309], [1344, 16, 1633], [4528, -217, -469], [-8490, 568, -622], [12878, 782, -1204], [-6532, 741, 71], [-2316, -705, -894], [1328, -1396, 1568], [14254, 278, -1791], [-1114, -283, 1169], [17642, -475, 60], [-8866, 316, -3322], [-27834, -432, 796], [6284, 544, 1244], [11874, -371, 986], [-17626, -474, 1388], [-7942, -207, -695], [20032, -1277, -269], [4528, 272, -1056], [-8548, -536, -880], [43964, -159, 801], [-10144, 638, -1266], [13992, -481, -954], [-9714, -1374, -1076], [-9196, 799, -84], [-1766, 1432, -523], [16422, -516, -1734], [26648, 1301, -425], [18628, 336, 1652], [-34128, -504, 461], [16902, -1754, 145], [21352, 1806, -1418], [-15794, -462, 2612], [6838, -630, 1784], [-11486, 338, -906], [-30518, 656, 1853], [27494, 1702, -948], [23724, -322, -1808], [28782, 1250, -1736], [-14564, -423, -2217], [19618, -1906, -568], [11260, 1108, 1578], [36300, 375, 546], [-23880, -706, -948], [10926, -761, -1309], [-26332, 1075, 151], [17972, 68, 2436], [42530, 662, 1216], [-38100, 1623, 1080], [-5626, 841, -4116], [13662, -1344, -2270], [16622, 1982, -1694], [15956, -947, 763], [-12666, 106, -2736], [28138, 678, 2111], [-12686, 2002, -1770], [5518, 1538, -2295], [-31708, -1256, 2528], [424, -1238, 11], [11076, -420, 774], [-27526, 249, 1029], [24170, -1814, -3082], [27386, 1762, -2732], [29540, 1695, 976], [22408, -2164, -2796], [8304, -1515, -1410], [-3044, 1391, 2181], [-12054, 840, -4108], [19898, 1368, 564], [26166, 707, -1028], [3990, 866, -1320], [-29196, -1522, -1384], [760, 20, 3156], [-22856, -796, -2367], [5244, 856, 1576], [-47110, -1713, 2677], [31110, 2716, -3830], [15480, 1298, 3666], [8882, 1215, 1454], [-2042, 432, 2239], [16238, 1283, -3779], [-8648, 750, -5306], [1286, 2783, -3135], [-5490, 1281, -2048], [32220, -293, -1046], [-30746, -1798, 2706], [-49216, -576, 636], [-32576, 1798, -3673], [-48196, 886, -68], [-30020, -2403, 1667], [21374, -676, 2092], [28062, -2609, -859], [-25508, 610, -5432], [-35444, -1349, -907], [2202, 1146, 4439], [31654, -1844, 3844], [-6412, 1310, 82], [26068, -1557, -1065], [-9902, -3426, 502], [16724, 1358, -3178], [-6278, 1858, -4321], [20754, -3591, -1399], [28604, -2817, -2290], [18364, 1132, 4318], [46824, 1128, -1224], [-55134, 1509, 652], [2964, 1240, -4188], [-49836, -453, -2383], [7982, 2346, -412], [420, 1547, 3580], [41132, 2314, -3952], [-14184, -1518, 474], [-17218, -3333, -1040], [-44424, 2830, -530], [49856, 387, 2113], [-26930, -1475, -887], [7192, 903, -4574], [47146, -2341, 2400], [-3834, 3610, -1620], [-19662, -1302, -1478], [-11282, -1168, -2145], [-71654, -936, 862], [-25296, -4334, 4240], [29574, -42, 8592], [6406, 4665, -3793], [-19936, 812, 3724], [-69356, 1590, -2100], [-46870, 3768, 1260], [-32606, -1173, 197], [-58412, 109, -3978], [-52372, -954, -2234], [-83456, -2237, 3440], [4080, -2143, -4168], [19076, 4518, -3404], [-65326, 3056, 1616], [-12694, -2016, -2464], [-54244, 1096, 166], [-47246, 4223, 535], [23840, -1416, -7024], [-68218, 1584, -930], [71232, -1144, -3236], [-16814, -2430, 4215], [-13782, -1297, -4927], [-70038, 90, 142], [18632, 730, -160], [-34176, -1336, 2900], [27532, -3773, 4955], [26616, -2880, 1164], [81846, -1324, -3014], [-30240, -913, 5407], [71200, -600, -796], [30898, -3439, 1239], [-30748, 2533, -268], [43660, 522, 252], [7834, -3745, 6687], [39218, -1543, 5559], [-24240, 3119, 5243], [37960, 4366, 1375], [-22274, -3229, -1848], [34278, 3276, -6608], [-81490, 168, -746], [106944, 270, 772], [-51386, -1670, 6068], [35536, 407, 5263], [104, -3271, 6861], [-30686, 1698, 6162], [59464, -3611, -1130], [42600, -1150, 4822], [19740, -2114, 4436], [-19784, 37, -8291], [100294, 12, 2634], [45404, -1690, -5476], [67562, 349, -6689], [39618, 5081, -2962], [46042, -3134, 32], [87804, -2002, -460], [58928, -2199, 2474], [105606, 1168, 78], [28544, -4461, -3373], [-314, -5685, 3422], [42366, -1232, -5562], [49910, 1587, 4968], [-55982, -85, 5064], [95042, -1339, 5985], [5258, -5416, 32], [-51564, 3046, -9798], [-52462, -854, 4642], [-24698, -2772, 7078], [127560, 287, -635], [79936, -3186, -6077], [20126, -3358, -6248], [-95016, 452, -9920], [113856, -1229, 4887], [-93772, -2205, 2311], [-1194, 1191, -5073], [74294, 1345, 7217], [-1362, 4275, -37], [-63368, -3474, -596], [-125310, -1249, -2146], [3200, 5896, 1268], [96810, 3191, -1566], [41370, 4507, 6177], [-15712, -4637, 6090], [66494, 584, 9804], [-72276, -1781, 5917], [39638, 26, -9144], [-49978, 5716, -2442], [-48984, 606, -10821], [-792, -1258, -1638], [114736, 1487, -3000], [29596, 692, -6798], [45910, -4537, 8383], [-87480, 2518, -2266], [34926, -6785, 4016], [-76570, 2098, 3820], [-64572, -276, -7228], [88206, -1988, -6634], [-104076, -877, -4486], [142672, 1982, -328], [60978, 1750, 2044], [-71742, -491, 1301], [-6430, 852, -8915], [-1866, -6603, 2774], [43694, -4410, 728], [25826, 3707, 9716], [-6446, -4271, -3101], [42158, -2441, -7251], [-71232, -3335, 3327], [-65586, -7577, 4252], [-11236, 526, -4054], [28602, -2324, 6948], [-123876, 246, -1699], [-130680, -563, 507], [61558, -2391, -6087], [-85928, -3514, -2214], [-58980, 2360, -1624], [26086, -291, 3772], [-44462, 823, -8975], [82076, 6369, -3045], [42958, -2418, -10494], [-81438, 1233, 880], [100516, -3375, -415], [66350, -3917, 12125], [62694, -1985, -5087], [-37974, 4168, -14160], [117948, 4967, -4972], [-140108, -1174, -4595], [-90052, -2141, -6513], [32620, 739, 3964], [-23000, -4218, 5064], [-31082, -904, -3249], [-59252, -6493, 9511], [-54274, 5181, -1047], [80648, 1424, 5172], [146548, -1172, -3298], [-36426, -54, -3760], [-45612, 127, 560], [1058, -1751, 5016], [-69356, 2532, -16188], [-86642, 1841, -1056], [-93738, 1666, -6576], [90608, -2298, 990], [-1650, 523, 3237], [105440, -4929, -4977], [-110032, 491, 2136], [36534, -7348, 6806], [123818, 2788, 1452], [100548, 1841, -10147], [-21732, -1203, 3222], [-104032, -3555, 1808], [-12788, -1777, 11969], [48506, 1004, -12024], [-118232, 6346, 1631], [-30712, 767, -3409], [148370, -2315, 2544], [-97422, 748, 1360], [-44462, -4625, -10027], [1908, -5310, 8832], [-34704, -1151, -10819], [-48754, -4559, 3629], [10070, -2872, 6394], [152854, 2807, -6198], [-44028, 8497, -6609], [10812, 5587, 2996], [-175148, -4342, 5644], [175558, -1045, -2481], [-196498, 2470, 1740], [91918, 769, -7216], [-88432, -5602, 5096], [-68986, 297, 2783], [7868, -609, 12560], [126776, 5818, -126], [67524, -5449, 3624], [2868, 7425, 7051], [-52864, -4124, 12012]]
